if(!dojo._hasResource["com.ibm.domUtilities"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.domUtilities"] = true;
/*
 * @author Ben
 */
dojo.provide( "com.ibm.domUtilities" );
/*
 * This module provides cross-browser functions for interacting with a DOM object to
 * create nodes and get and set attributes that may or may not be namespaced.  Since
 * browsers use various functions for interacting with namespaced DOM nodes and attributes,
 * this module abstracts those away and provides a single layer for using those operations.
 */

dojo.require( "dojox.xml.parser" );
dojo.require( "com.ibm.portal.xslt" );

(function(){
	var dUtil = com.ibm.domUtilities = {
		constants: {
			// NodeTypes: Constants for defining integer values for node type strings
			NodeTypes: {
				element: 1,
				attribute: 2,
				text: 3,
				cdata: 4,
				entity_reference: 5,
				entity: 6,
				processing_instruction: 7,
				comment: 8,
				document: 9,
				document_type: 10,
				document_fragment: 11,
				notation: 12
			}
		},
		
		nsInfo: function(/*String*/ str, /*Object*/ namespaces) {
			// summary: Extracts the namespace URI, prefix, and local name of a QName according to this
			//		object's namespaces map that includes known prefixes and their namespace URIs.  If
			//		the string is not namespaced, then the namespaceURI and prefix are set to null, and
			//		the localName is set to the str as-is.
			// str: QName to extract namespace from
			// namespaces: Map of namespaces where keys are namespace prefixes and values are the namespace URIs.
			// returns: Map - {namespaceURI: namespaceURI, prefix: prefix, localName: localName}
			var parts = str.split(":");
			var prefix, localName;
			if(parts.length > 1) {
				prefix = parts[0];
				localName = parts[1];
				return { namespaceURI: namespaces[prefix], prefix: prefix, localName: localName};
			}
			else {
				return { namespaceURI: null, prefix: null, localName: str};
			}
		},
		getAttribute: function(/*DOMNode*/ node, /*String*/ attrName, /*Object*/ namespaces) {
			// summary: Gets an attribute of the given node, supporting namespaced and 
			// 		non-namespaced attributes.
			// node: Node to retrieve the attribute value from
			// attrName: The name of the attribute to get from the node.
			//		If the attribute is namespaced, it will be split into its parts using the : delimiter
			//		and the namespace used during lookup will be the one keyed in the namespace map.
			// namespaces: Map of prefixes to namespaceURIs
			// returns: Object value of attribute
			var ret = null;
			var parts = dUtil.nsInfo(attrName, namespaces);
			if(parts.namespaceURI) {
				if (node.getAttributeNS) {
					ret = node.getAttributeNS(parts.namespaceURI, parts.localName);
				}
				else 
					if (dojo.isIE) {
						ret = node.attributes.getQualifiedItem(parts.localName, parts.namespaceURI);
						if (ret) {
							ret = ret.value;
						}
					}
					else {
						ret = node.getAttribute(attrName);
					}
			}
			else {
				ret = node.getAttribute( attrName );
			}
			return ret;
		},
		setAttribute: function(/*DOMNode*/ node, /*String*/ attrName, /*String*/ value, /*Object*/ namespaces) {
			// summary: Sets an attribute of the given node, supporting namespaced and 
			// 		non-namespaced attributes.
			// node: Node to set the attribute value on
			// attrName: The name of the attribute to set on the node.
			//		If the attribute is namespaced, it will be split into its parts using the : delimiter
			//		and the namespace used during lookup will be the one keyed in the namespace map.
			// value: The actual value to store in the node at the key attribute specified by attrName.
			// namespaces: Map of prefixes to namespaceURIs
			if (value === true) {
				value = "true";
			}
			else {
				if (value === false) {
					value = "false";
				}
				else 
					if (value === null) {
						value = "";
					}
			}
			var parts = dUtil.nsInfo(attrName, namespaces);
			if (parts.namespaceURI) {
				if (node.setAttributeNS) {
					node.setAttributeNS(parts.namespaceURI, attrName, value);
				}
				else 
					if (dojo.isIE && node.ownerDocument) {
						var attrNode = node.ownerDocument.createNode(dUtil.constants.NodeTypes.attribute, attrName, parts.namespaceURI);
						attrNode.value = value;
						node.setAttributeNode(attrNode);
					}
					else {
						node.setAttribute(attrName, value);
					}
			}
			else {
				node.setAttribute(attrName, value);
			}
		},
		removeAttribute: function(/*DOMNode*/ node, /*String*/ attrName, /*Object*/ namespaces) {
			// summary: Removes an attribute of the given node, supporting namespaced and 
			// 		non-namespaced attributes.
			// node: Node to remove the attribute from
			// attrName: The name of the attribute to remove from the node.
			//		If the attribute is namespaced, it will be split into its parts using the : delimiter
			//		and the namespace used during lookup will be the one keyed in the namespaces map.
			// namespaces: Map of prefixes to namespaceURIs
			var parts = dUtil.nsInfo(attrName, namespaces);
			if(parts.namespaceURI) {
				if(node.removeAttributeNS) {
					// for some reason we have to use the localName when getting the attribute even
					// though we have to use the full name when setting it
					node.removeAttributeNS( parts.namespaceURI, attrName );
				}
				else if(dojo.isIE) {
					node.attributes.removeQualifiedItem( parts.localName, parts.namespaceURI );
				}
				else node.removeAttribute( attrName );
			}
	        else node.removeAttribute( attrName );
		},
		hasAttribute: function(/*DOMNode*/ node, /*String*/ attrName, /*Object*/ namespaces) {
			// summary: Checks if an attribute of the given node exists, supporting namespaced and 
			// 		non-namespaced attributes.
			// node: Node to check for the existence of the attribute from
			// attrName: The name of the attribute to check from the node.
			//		If the attribute is namespaced, it will be split into its parts using the : delimiter
			//		and the namespace used during lookup will be the one keyed in this item or store's 
			// 		namespace map.
			// namespaces: Map of prefixes to namespaceURIs
			// returns: Boolean true if attribute exists, false otherwise
			var ret = null;
			var parts = dUtil.nsInfo(attrName, namespaces);
			if (parts.namespaceURI) {
				if (node.hasAttributeNS) {
					ret = node.hasAttributeNS(parts.namespaceURI, parts.localName);
				}
				else {
					if (dojo.isIE) {
						ret = node.attributes.getQualifiedItem(parts.localName, parts.namespaceURI) !== null;
					}
					else {
						ret = node.hasAttribute(attrName);
					}
				}
			}
			else {
				if (!dojo.isIE) {
					ret = node.hasAttribute(attrName);
				}
				else {
					ret = node.getAttributeNode(attrName) !== null;
				}
			}
			
			return ret;	
		},
		createElement: function(/*DOMDoc*/ doc, /*String*/ tagName, /*Object*/ namespaces, /*Object?*/ attrMap) {
			// summary: Cross-browser implementation for creating a namespaced node using the namespace
			//		map in this object to resolve prefixes to namespace URIs.
			//		If the tagName is not namespaced or a namespace URI cannot be resolved
			// 		using this object's namespaces map, a non-namespaced element will be created instead.
			// doc: DOMDoc document object to use to create the DOM node.
			// tagName: String name for the tag name of the element to create.  
			// 		Can be namespaced like prefix:localName or not like localName.
			// namespaces: Map of namespaces where keys are namespace prefixes and values are the namespace URIs.
			// attrMap: Optional argument specifying a key-value map of attributes to be
			// 		set on the node before returning it. 
			// returns: DOMNode A new DOM node that was created by doc
			var node = null;
			var parts = dUtil.nsInfo(tagName, namespaces);
			if(parts.namespaceURI) {
				if(doc.createElementNS) {
					// W3C standard way of creating namespaced elements
					node = doc.createElementNS(parts.namespaceURI, tagName);
				}
				else if(dojo.isIE) {
					// we have to check dojo.isIE because IE throws an error when trying to check for
					// the existence of particular methods on the document object
					
					// this is IE's way of creating a namespaced node
					// use the tagName since it already includes the prefix as well
					node = doc.createNode("element", tagName, parts.namespaceURI);
				}
			}
			// if we don't have a node at this point, just create it without a namespace
			if (!node) {
				node = doc.createElement(tagName);
			}
			
			for(var name in attrMap) {
				dUtil.setAttribute(node, name, attrMap[name], namespaces);
			}
			
			return node;
		},
		createFromJson: function(/*DOMDoc*/ doc, /*Object*/ def, /*Object*/ namespaces, /*DOMNode?*/ parentNode) {
			// summary: Creates a new element using a JSON object as a declarative definition of the structure
			//		to create.  Can pass in an optional parentNode argument to append the new element as a child
			//		of the parentNode.
			// doc: Document object to use for creating the DOM nodes and that will own the created element.
			// def: Object JSON object DOM definition specifying the structure to create.  Supports these properties -
			//		{
			//			name: String name of the element to create, can be namespace-prefixed using a prefix in the
			//				namespaces map argument
			//			attributes: Object map of attributes to set on the element with names and values which may 
			// 				be namespace-prefixed names
			//			children: Array of JSON object DOM definitions to allow creating nested structures with the
			//				same syntax
			//		}
			// namespaces: Object map of prefix-namespaceURI pairs to use for namespace resolution of elements and
			// 		attributes created by this function.
			// parentNode: DOMNode optional argument to specify the parent of the newly created element
			// returns: DOMNode the created element	
			var obj = null;
			if(!dojo.isObject(def)){
				obj =  doc.createTextNode(def);
				if (parentNode) {
					parentNode.appendChild(obj); // append the nodes to the DOM as they are created to avoid memory issues in IE
				}
			}
			else {
				obj = dUtil.createElement(doc, def.name, namespaces, def.attributes);
				if (parentNode) {
					parentNode.appendChild(obj); // append the nodes to the DOM as they are created to avoid memory issues in IE
				}
				dojo.forEach(def.children, function(childDef) {
					dUtil.createFromJson(doc, childDef, namespaces, obj);
				});
			}
			return obj;
		},
		removeChildren: function(node) {
			// summary: Removes all the children from a node and returns a NodeList of those orphaned children
			//		which may consist of children of different node types.
			// node: DOMNode to remove children from
			var arr = [];
			while(node.hasChildNodes()) {
				arr.push(node.removeChild(node.firstChild));
			}
			
			// use official ctor for dojo 17 compat
			var nl = new dojo.NodeList();
			
			for (var i=0; i<arr.length; ++i) {
			    nl.push(arr[i]);
			}
			
			return nl;
		},
		textContent: function(/*DOMNode*/ node, /*String?*/ text) {
			// summary: Gets or sets the text content under a given DOM node.  This acts as a getter if 
			// 		one argument is passed, while it acts as a setter if two arguments are passed.
			// node: DOMNode to get/set the text content for
			// text: String optional argument specifying the text to set under the node, replacing the existing
			// 		text. 
			return dojox.xml.parser.textContent.apply(null, arguments);
		},
		innerXML: function(/*DOMNode*/ node) {
			// summary: Serializes a DOM node into a string representation of the DOM at that level (including the
			//		node itself).
			// node: DOMNode node to serialize
			return dojox.xml.parser.innerXML(node);
		},
		stringFromDoc: function(/*DOMNode*/ node) {
			// summary: Serializes a DOM node into a string representation of the DOM at that level (including the
			//		node itself).  Same as innerXML.
			// node: DOMNode node to serialize
			return dUtil.innerXML(node);
		},
		docFromString: function(/*String*/ str) {
			// summary: Deserializes/parses a string representation of a DOM document into a new DOM document object.
			// str: String to deserialize/parse
			return com.ibm.mm.enabler.utils.Dom.createDocument(str && str.length > 0 ? str : null);
			//return com.ibm.portal.xslt.loadXmlString(str && str.length > 0 ? str : null);
		},
		encodeXML: function(/*String*/ xmlStr, /*Boolean?*/ encodeQuotes) {
			// summary: Encodes a string for inclusion into an XML document so that any XML delimiters will
			//		be encoded as CDATA.
			// xmlStr: String value to encode
			// encodeQuotes: Boolean optional argument indicating whether or not quotes should be encoded as
			//		well.  By default, single and double quotes are not encoded in the returned string for
			//		increased flexibility depending on the context in which the caller uses the encoded string.
			if (dojo.isString(xmlStr)) {
				xmlStr = xmlStr.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;");
				if (encodeQuotes) {
					xmlStr = xmlStr.replace(/'/gm, "&#039;").replace(/"/gm, "&#034;");
				}
				return xmlStr;
			}
			else {
				if (xmlStr !== null) {
					return xmlStr;
				}
				else {
					return "";
				}
			}
		},
		decodeXML: function(/*String*/ xmlStr, /*Boolean?*/ decodeQuotes) {
			// summary: Decodes an XML-encoded string so that any XML-encoded characters will be decoded
			//		to their non-encoded value.
			// xmlStr: String value to decode
			// decodeQuotes: Boolean optional argument indicating whether or not quotes should be decoded as
			//		well.  By default, single and double quotes are not decoded in the returned string for
			//		increased flexibility depending on the context in which the caller uses the decoded string.
			if (dojo.isString(xmlStr)) {
				xmlStr = xmlStr.replace(/&lt;/gm, "<").replace(/&gt;/gm, ">").replace(/&amp;/gm, "&");
				if (decodeQuotes) {
					xmlStr = xmlStr.replace(/&#039;/gm, "\'").replace(/&#034;/gm, "\"");
				}
				return xmlStr;
			}
			else {
				if (xmlStr != null) {
					return xmlStr;
				}
				else {
					return "";
				}
			}
		}
	};
	
	dUtil.constants.MSXML_NodeTypes = dUtil.constants.NodeTypes;	// backwards compat
})();


}
