/* Copyright IBM Corp. 2015  All Rights Reserved.                    */
dojo.provide( "com.ibm.mashups.enabler.EmbeddingGenerator" );  

dojo.require("com.ibm.mashups.services.ServiceManager");
dojo.require("com.ibm.mashups.enabler.services.ConfigService");
dojo.require("com.ibm.mashups.enabler.services.ConfigConstants");
dojo.require("com.ibm.mashups.enabler.utils.EndpointHelper");

/**
 * This class allows to generate single line script include for the embedding feature
 * @ibm-spi
 * @ibm-module Embedding
 */
dojo.declare("com.ibm.mashups.enabler.EmbeddingGenerator", null, {
    widgetDefinition: "",
    pid: "",
    width: "",
    height: "",
    scrollbar: false,
    sandboxed: false,
    framework: "light",
    preferences: null,
    
    /**
     * This is the constructor for EmbeddingGenerator, which allows to instantiate the EmbeddingGenerator with the default settings for widget / page embedding.
     * @param {String} widgetDef the path to the widget definition (either absolute or relative to the embedding server). In case of page embedding widgetDef points to the layout widget to use for page embedding
     * @param {String} pid the page ID of the page to include. If only widget embedding is needed, this parameter can be left empty.
     */
    constructor: function(widgetDef, pid) {
        this.widgetDefinition = widgetDef;
        this.pid = pid;
        this.hostAlias = null;
    },
    /**
     * This method will get a hostname, in sandboxed mode it will be a magic host name. 
     * NOTE: the host name is the same for the lifetime of this object.
     */
    _getSecureHostAlias: function() {
        if (this.hostAlias === null) {
            this.hostAlias = window.location.host;
        }
        
        return this.hostAlias;
    },

    /**
     * Creates the single script include with the current attribute settings of this class.
     */
    toString: function() {
        var url = window.location.protocol;
        url += "//" + this._getSecureHostAlias();
        url += com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME).getValue(com.ibm.mashups.enabler.services.ConfigConstants.CONTEXT_ROOT);
        
        // add the contenthandler to use
        url += com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME).getValue(com.ibm.mashups.enabler.services.ConfigConstants.EMBEDDING_PUBLIC);
        
        url += "?";
        
        // specify the uri schema
        url += "uri=widget:js";
        
        // add the url parameter, only applies to widget embedding
        if (null !== this.widgetDefinition) {
            var resolvedUrl = com.ibm.mashups.enabler.utils.EndpointHelper.resolve(this.widgetDefinition);
            url += "&url=" + escape(resolvedUrl);
        }
        
        if (this.pid) {
            url += "&pid=" + escape(this.pid);
            
            this.height = this.height ? this.height:"850";
            this.width  = this.width  ? this.width:"750";
        } else {
            this.height = this.height ? this.height:"500";
            this.width  = this.width  ? this.width:"400";
        }
            
        url += "&h=" + escape(this.height);
        url += "&w=" + escape(this.width);
        
        if (this.scrollbar) {
            url += "&sb=yes";
        }
        else {
            url += "&sb=no";
        }
        if (this.framework && this.framework != "light") {
            url += "&fw=" + this.framework;
        }

        if (this.sandboxed) {
            url += "&sandbox=yes";
        }

        if (this.preferences && this.preferences.getAllNames()) {
            var names = this.preferences.getAllNames();
            
            for (var i=0, l = names.length; i<l; ++i) {
                url += "&p_" + encodeURIComponent(names[i]) + "=" + encodeURIComponent(this.preferences.getItemValue(names[i]));
            }
        }
        
        return url;
    },

    /**
     * gets the current widget definition path for this embedding generator.
     * @type String
     */
    getWidgetDefinition: function() {
        return this.widgetDefinition;
    },
    /**
     * sets the current widget definition path for this embedding generator
     * @param {String} value the widget definition url
     */
    setWidgetDefinition: function(value) {
        this.widgetDefinition = value;
    },
    
    /**
     * gets the current page id for this embedding generator.
     * @type String
     */
    getPid: function() {
        return this.pid;
    },
    
    /**
     * sets the pid for this embedding generator
     * @param {String} value the pid
     */
    setPid: function(value) {
        this.pid = value;
    },

    /**
     * gets the current width for this embedding generator.
     * @type int
     */
    getWidth: function() {
        return this.width;
    },
    
    /**
     * sets the current width for this embedding generator
     * @param {int} value the width
     */
    setWidth: function(value) {
        this.width = value;
    },
    
    /**
     * gets the current height for this embedding generator.
     * @type int
     */
    getHeight: function() {
        return this.height;
    },
    /**
     * sets the current height for this embedding generator
     * @param {int} value the height
     */
     setHeight: function(value) {
        this.height = value;
    },

    /**
     * gets the current  state of scrollbar for this embedding generator.
     * @type boolean
     */
    getScrollbar: function() {
        return this.scrollbar;
    },
    /**
     * sets whether the scrollbar should be enabled or not
     * @param {boolean} value true when the scrollbar should be enabled, default: false
     */
    setScrollbar: function(value) {
        this.scrollbar = value;
    },

    /**
     * gets whether the widget should be sandboxed or not
     * @type boolean
     */
    getSandboxed: function() {
        return this.sandboxed;
    },
    /**
     * sets whether the widget should be sandboxed
     * @param {boolean} value true when the widget should be sandboxed, default: false
     */
    setSandboxed: function(value) {
        this.sandboxed = value;
    },

    /**
     * gets the current framework typefor this embedding generator.
     * @type String
     */
    getFramework: function() {
        return this.framework;
    },
    
    /**
     * sets the current framework for this embedding generator
     * @param {String} value the framework to use for the widget (not used in page mode), default: to false
     */
    setFramework: function(value) {
        this.framework = value;
    },
    
    /**
     * gets the current framework typefor this embedding generator.
     * @type ItemSet
     */
    getPreferences: function() {
        return this.preferences;
    },
    
    /**
     * sets the preferences for this embedding generator
     * @param {ItemSet} value the preferences
     */
    setPreferences: function(value) {
        this.preferences = value;
    }
});
